--=======================================================================
--miniSIPServer operator break-in service
--
--Copyright (C) MYVOIPAPP,Inc. All Rights Reserved.
--
--author: Gilson
--
--History:
--2016-11-01 created.
--=======================================================================
require "services"
require "utils.data_string"
require "utils.dba"

--service states
local EBREAKIN_STATE_IDLE = 0

--====================================
--
-- Cache DB functions
--
--====================================
local function cdbGetPeerSSF(profileID)
    TraceInfo("\t cdbGetPeerSSF")
    local sqlStr=string.format("select fsmID from tbl_ccf where "..
                 "profileID=%d and callStage=%d limit 1", profileID, ECALL_TALK)
    local result = cdbQuery(sqlStr)    
    if result~=MSS_ERR_OBJ_FOUND then
        TraceInfo("\t\t cannot find target call from cache.")
        return -1
    end

    fsm.peerSSFID = cdbGetRowIntVal(0)
    TraceInfo("\t\t find target SSF ID: "..IntToHexStr(fsm.peerSSFID))
    return 1             
end


local function cdbIsCallInConf(ccfID)
    local sqlStr=string.format("select count(*) from tbl_conference where "..
                 "ccfID=%d", ccfID)
    local result = cdbQuery(sqlStr)
    if result~=MSS_ERR_OBJ_FOUND or 0==cdbGetRowIntVal(0) then
        return 0
    else
        return -1
    end                 
end

--====================================
--
-- FSM common functions
--
--====================================
local function initParams()
    fsm.breakInPrefix = ssfGetSysVarOperatorBreakIn()
    fsm.peerSSFID = MSS_INVALID_OBJ_ID
    fsm.targetUserNbr=""
    fsm.targetUserProfileID=MSS_INVALID_OBJ_ID
end

local function checkServiceRight()
    if ssfUserCanBreakIn(fsm.profileID)==MSS_YES then
        return 1
    else
        TraceInfo("\t the user doesn't have break-in right.")
        return -1
    end
end

local function getTargetUserInfo()
    TraceInfo("\t getTargetUserInfo")
    fsm.targetUserNbr = StrLstrip(fsm.calledNbr, fsm.breakInPrefix)
    TraceInfo("\t\t target user: "..fsm.targetUserNbr)

    fsm.targetUserProfileID = ssfGetUserProfileID(fsm.targetUserNbr)
end


local function checkTargetUser()
    TraceInfo("\t checkTargetUser")
    if fsm.targetUserProfileID==MSS_INVALID_OBJ_ID then
        TraceInfo("\t\t cannot find target profileID") 
        return -1 
    end

    local result = userHasOverstep(fsm.targetUserNbr)
    if result==MSS_YES then
        TraceInfo("\t\t the target user has 'overstep' right.")
        return -2
    end

    return 0
end

local function exceptionProc()
    TraceInfo("\t exceptionProc")
    SendReleaseCall()
    EndService()
end

local function joinCall()
    SendJoinCall(fsm.peerSSFID)    
end
--====================================
--
-- FSM state functions
--
--====================================
local function onIdleProc()
    TraceInfo("breakin: onIdleProc")
    initParams()
    
    local result = checkServiceRight()
    if result<0 then
        exceptionProc()
        return
    end

    getTargetUserInfo()
    result = checkTargetUser()
    if result<0 then
        exceptionProc()
        return
    end

    result = cdbGetPeerSSF(fsm.targetUserProfileID)
    if result<0 then
        exceptionProc()
        return 
    end

    result = cdbIsCallInConf(fsm.peerSSFID)
    if result<0 then
        TraceInfo("\t target user is in conference, reject current operation.")
        exceptionProc()
        return
    end

    joinCall()
    EndService()
end


--====================================
--
-- Service main function
--
--====================================
function DestroyProc()
    TraceInfo("DestroyProc")
    -- do nothing
end

function MainProc(currMsg)
    if fsm.state==EBREAKIN_STATE_IDLE then
        onIdleProc()

    else
        TraceInfo("unknown state: "..fsm.state)
        exceptionProc()
    end
end

