--=======================================================================
--miniSIPServer call-retrieve service script
--
--Copyright (C) MYVOIPAPP,Inc. All Rights Reserved.
--
--Author: Gilson
--
--History:
--2014-02-27 Migrated from MSS core.
--2016-09-14 Migrated from python script.
--=======================================================================
require "services"
require "utils.data_string"
require "utils.dba"

-- service states
local ECALL_RETRIEVE_IDLE = 0
local ECALL_RETRIEVE_WAIT_RESP = 1


-- Timer values
local CR_TIMER_WAIT_RESP = 10

-- call park and retrieve
local MIN_CALLPARK_CHANNEL = 1

--====================================
--
-- Cache DB functions
--
--====================================
local function cdbCallParkGet()
    local sqlStr=string.format("select callerSRFID, ssfID, scfID from tbl_callpark "..
                 "where channel=%d", fsm.channel)
    local result=cdbQuery(sqlStr)
    if result~=MSS_ERR_OBJ_FOUND then
        TraceInfo("\t\t Fail. SQL: "..sqlStr)
        return -1 
    end

    fsm.peerSRFID = cdbGetRowIntVal(0)
    fsm.objSspFsmID = cdbGetRowIntVal(1)
    fsm.objScpFsmID = cdbGetRowIntVal(2)
    return 1                  
end

--====================================
--
-- FSM common functions
--
--====================================
local function initParams()
    fsm.callRetrievePrefix = ssfGetSysVarCallRetrievePrefix()
    fsm.channel = 0
    fsm.objScpFsmID = 0    --target SCP FSM to be retrieved
    fsm.objSspFsmID = 0
    fsm.peerSRFID = 0    
end


local function procException()
    TraceInfo("\t procException")
    SendReleaseCall()
    EndService()
end


local function saveIDPInfo()
    TraceInfo("\t saveIDPInfo")
    local servNbr = fsm.calledNbr
    if StrHasPrefix(servNbr, fsm.callRetrievePrefix)==MSS_FALSE then return -1 end

    local channelStr = StrLstrip(servNbr, fsm.callRetrievePrefix)
    channelStr = StrRstrip(channelStr, "#")
    fsm.channel = ToInt(channelStr)
    if fsm.channel<MIN_CALLPARK_CHANNEL then return -2 end

    TraceInfo("\t\t prepare to retrieve channel "..fsm.channel)
    return 1
end


local function bridge2Parties()
    TraceInfo("\t bridge2Parties")
    local b2pArg=CreateBridge2Parties()
    b2pArg.peerSrfID = fsm.peerSRFID
    b2pArg.peerSsfFsmID = fsm.objSspFsmID
    SendBridge2Parties(b2pArg)
end


--====================================
--
-- FSM state functions
--
--====================================
local function onIdleProc()
    TraceInfo("callRetrieve: onIdleProc")
    initParams()

    local result = saveIDPInfo()
    if result<0 then
        TraceInfo("\t fail to save IDP information. errCode="..result)
        procException()
        return 
    end

    result = cdbCallParkGet()
    if result<0 then
        TraceInfo("\t fail to get park information.")
        procException()
        return
    end

    SendCallRetrieveReq(fsm.objScpFsmID)

    StartTimer(CR_TIMER_WAIT_RESP)
    EnterState(ECALL_RETRIEVE_WAIT_RESP)
end


local function onWaitRespProc(currMsg)
    TraceInfo("onWaitRespProc")
    if currMsg~=ESCP_EVT_SCF_CALL_RETRIEVE_RESP then
        procException()
        return 
    end

    StopTimer()
    bridge2Parties()
    SendTransClose()
    EndService()
end

--====================================
--
-- Service main function
--
--====================================
function DestroyProc()
    TraceInfo("DestroyProc")
    -- do nothing
end


function MainProc(currMsg)
    if fsm.state==ECALL_RETRIEVE_IDLE then
        onIdleProc()

    elseif fsm.state==ECALL_RETRIEVE_WAIT_RESP then
        onWaitRespProc(currMsg)

    else
        TraceInfo("unknown state: "..fsm.state)
        procException()
    end
end