require "services.ssf_def"

-- process result.
PRC_PROCESSED  = 1
PRC_DISCARD    = 2
PRC_KILL       = 3
PRC_SAVE       = 10

-- MSS core records details of current message
currMessage={}

--
--Service FSM definitions.
--

fsm={
    -- service datas
    calledNbr="",
    callerNbr="",
    redirNbr="",
    origDialPlan="",
    serviceNbr="",
    callerSrfID=0,
    mediaDir=0,
    foSSPInd=0,
        
    -- special service information indicators
    freeCallInd=0,
    suppressCallLevel=0,
    
    -- basic data
    profileID=0,
    scpFsmID=0,
    tracedInd=0,    
    state=0,
    procResult=0,    
    
    -- details of current message
    event=0,
    msgType=0,
    orgTask=0,
    orgFsmType=0,
    orgFsmID=0
}

-- ===========================================================
--
-- FSM functions
-- 
-- ===========================================================
function TraceInfo(infoVal)
    if MSS_YES==fsm.tracedInd then
        ssfTraceInfo(fsm.scpFsmID, infoVal)
    end
end

function EndService()
    TraceInfo("\t EndService")
    fsm.procResult = PRC_KILL
end

function SaveMessage()
    TraceInfo("\t SaveMessage")
    fsm.procResult = PRC_SAVE
end

function HandleException()
    print(debug.traceback())
    SendReleaseCall()
    EndService()
end

function EnterState(new_state)
    TraceInfo("\t Enter state: from "..fsm.state .. " to "..new_state)
    fsm.state = new_state
end

function StartTimer(seconds)
    TraceInfo("\t start timer "..seconds.." seconds.")
    ssfStartTimer(fsm.scpFsmID, seconds)
end

function StopTimer()
    TraceInfo("\t stop timer")
    ssfStopTimer(fsm.scpFsmID)
end

function GetSsfFsmID(ssfIndex)
    if ssfIndex==nil then ssfIndex=0 end
    return ssfGetSsfFsmID(fsm.scpFsmID, ssfIndex)
end

-- =============================================================
-- 
-- Functions to send messages to SSF.
-- 
-- =============================================================
function SendUDPString(destIP, destPort, udpString)
    TraceInfo("\t SendUDPString")
    ssfSendUDPString(destIP, destPort, udpString)
end

function SendTransClose(ssfindex)
    TraceInfo("\t SendTransClose")
    if ssfIndex==nil then ssfIndex=0 end
    ssfSendTransClose(fsm.scpFsmID, ssfIndex)
end

function SendContinue(ssfIndex)
    TraceInfo("\t SendContinue")
    if ssfIndex==nil then ssfIndex=0 end
    ssfSendContinue(fsm.scpFsmID, ssfIndex)
end

function SendReconnect(ssfIndex)
    TraceInfo("\t SendReconnect")
    if ssfIndex==nil then ssfIndex=0 end

    ssfSendReconnect(fsm.scpFsmID, ssfIndex)
end

function SendRRBE(leg_id, dp, monitor_type, ssfIndex)
    TraceInfo("\t SendRRBE")
    if ssfIndex==nil then ssfIndex=0 end

    ssfSendRRBE(fsm.scpFsmID, leg_id, dp, monitor_type, ssfIndex)
end

function SendDisarmAll(ssfIndex)
    TraceInfo("\t SendDisarmAll")
    if ssfIndex==nil then ssfIndex=0 end
    
    ssfSendDisarmAll(fsm.scpFsmID, ssfIndex)
end

function SendReleaseCall(cause, ssfIndex)
    TraceInfo("\t SendReleaseCall")
    if cause==nil then cause=16 end -- 16 is "normal clear"
    if ssfIndex==nil then ssfIndex=0 end
    TraceInfo("\t\t cause="..cause..",ssfIndex="..ssfIndex)
    
    ssfSendReleaseCall(fsm.scpFsmID, cause, ssfIndex)
end

function SendConnectWithArg(connArg, ssfIndex)
    TraceInfo("\t SendConnectWithArg")
    if ssfIndex==nil then ssfIndex=0 end

    connArg.scpFsmID = fsm.scpFsmID
    connArg.ssfIndex = ssfIndex

    ssfSendConnect(connArg)
end

function SendConnect(destNbr, ssfIndex)
    local connArg=CreateConnectArg()
    connArg.destNbr = destNbr

    SendConnectWithArg(connArg,ssfIndex)
end

function SendCTR(leg_id, ssfIndex)
    TraceInfo("\t SendCTR")
    if ssfIndex==nil then ssfIndex=0 end
    
    ssfSendCTR(fsm.scpFsmID, leg_id, ssfIndex)
end

function SendDFC(leg_id, ssfIndex)
    TraceInfo("\t SendDFC")
    if ssfIndex==nil then ssfIndex=0 end

    ssfSendDFC(fsm.scpFsmID, leg_id, ssfIndex)
end

-- paArg MUST be created by function 'CreatePA'
function SendPA(leg_id, paArg, ssfIndex)
    TraceInfo("\t SendPA")
    if ssfIndex==nil then ssfIndex=0 end

    paArg.scpFsmID = fsm.scpFsmID
    paArg.ssfIndex = ssfIndex
    paArg.legID = leg_id

    ssfSendPA(paArg)
end

function SendMusic(leg_id, annID, ssfIndex)
    local paArg = CreatePA()
    paArg.infoSend_annID_annType = ESRF_INFO_ANN_ID
    paArg.infoSend_annID_annID = annID
    paArg.infoSend_numOfRepeat = 0xFFFFFFFF
    paArg.infoSend_duration = 0

    SendPA(leg_id, paArg, ssfIndex)
end

function SendRingBackToneToCaller(ssfIndex)
    return SendMusic(CALLER_LEGID, ANN_ID_COMMON_RINGBACK, ssfIndex)
end

function SendMusicToCaller(ssfIndex)
    return SendMusic(CALLER_LEGID, ANN_ID_COMMON_MUSIC, ssfIndex)
end

-- pncArg MUST be created by function 'CreatePNC'
function SendPNCWithArg(leg_id, pncArg, ssfIndex)
    TraceInfo("\t SendPNCWithArg")
    if ssfIndex==nil then ssfIndex=0 end

    pncArg.scpFsmID = fsm.scpFsmID
    pncArg.ssfIndex = ssfIndex
    pncArg.legID = leg_id

    ssfSendPNC(pncArg)
end

function SendPNC(leg_id, ann_id, first_timeout, inter_timeout, ssfIndex)
    if ssfIndex==nil then ssfIndex=0 end

    local pncArg=CreatePNC()
    pncArg.infoSend_annID_annID = ann_id
    pncArg.infoSend_numOfRepeat = 1
    pncArg.firstDigitTimeout = first_timeout
    pncArg.intervalDigitTimeout = inter_timeout

    SendPNCWithArg(leg_id, pncArg, ssfIndex)
end

function SendRecordANN(leg_id, rdAnnArg, ssfIndex)
    TraceInfo("\t SendRecordANN")
    if ssfIndex==nil then ssfIndex=0 end

    rdAnnArg.scpFsmID = fsm.scpFsmID
    rdAnnArg.ssfIndex = ssfIndex
    rdAnnArg.legID = leg_id

    ssfSendRecordAnn(rdAnnArg)
end


function SendBridge2Parties(b2pArg, ssfIndex)
    if ssfIndex==nil then ssfIndex=0 end

    b2pArg.scpFsmID = fsm.scpFsmID
    b2pArg.ssfIndex = ssfIndex
    ssfSendBridge2Parties(b2pArg)
end


-- icaArg MUST be created by function 'CreateICA'
function SendICA(icaArg, ssfIndex)
    if ssfIndex==nil then ssfIndex=0 end

    icaArg.scpFsmID = fsm.scpFsmID
    icaArg.ssfIndex = ssfIndex
    ssfSendICA(icaArg)
end


function SendDisconnectLeg(legID, ssfIndex)
    TraceInfo("\t SendDisconnectLeg")
    if ssfIndex==nil then ssfIndex=0 end
    ssfSendDisconnectLeg(fsm.scpFsmID, legID, ssfIndex)
end


function SendJoinCall(peerSSFID, ssfIndex)
    TraceInfo("\t SendJoinCall")
    if ssfIndex==nil then ssfIndex=0 end
    ssfSendJoinCall(fsm.scpFsmID, peerSSFID, ssfIndex)
end


function SendPickUP(obcpID)
    TraceInfo("\t SendPickUP")
    ssfSendPickUP(fsm.scpFsmID, obcpID)
end


function SendCallRetrieveReq(objScpID)
    TraceInfo("\t SendCallRetrieveReq")
    ssfSendCallRetrieveReq(fsm.scpFsmID, objScpID)
end


function SendCallRetrievedResp(reqScpID)
    TraceInfo("\t SendCallRetrievedResp")
    ssfSendCallRetrieveResp(fsm.scpFsmID, reqScpID)
end

function SendCreateConf(legID, ssfIndex)
    TraceInfo("\t SendCreateConf")
    if ssfIndex==nil then ssfIndex=0 end
    ssfSendCreateConf(fsm.scpFsmID, legID, ssfIndex)
end

function SendAttendConf(legID, confNodeID, ssfIndex)
    TraceInfo("\t SendAttendConf")
    if ssfIndex==nil then ssfIndex=0 end
    ssfSendAttendConf(fsm.scpFsmID, legID, confNodeID, ssfIndex)
end

-- =============================================================
-- 
-- Functions to send messages to SCF.
-- 
-- =============================================================
function SendScfEnterConfRoom(peerScfID)
    TraceInfo("\t SendScfEnterConfRoom")
    ssfSendScfEnterConfRoom(fsm.scpFsmID, peerScfID)
end

-- EOF
