--=======================================================================
--miniSIPServer hunting-group-operator service script
--
--Copyright (C) MYVOIPAPP,Inc. All Rights Reserved.
--
--Author: Gilson
--
--History:
--2014-02-24 Migrated from MSS core.
--2016-09-12 Migrated from Python script.
--2018-01-11 Get HG operator infor from cache DB.
--=======================================================================
require "services"
require "utils.data_string"
require "utils.dba"

--HG operation type
local EHG_OP_UNKNOWN  = 0
local EHG_OP_LOGIN    = 1
local EHG_OP_LOGOUT   = 2

--HG operation state
local EHG_STATE_IDLE      = 0
local EHG_STATE_WAIT_ANN  = 1

--HG audio
local HGOP_ANN_ID_NOTICE = 0x01080001

--HG timer
local HGOP_TIMER_WAIT_ANN = 30


--====================================
--
-- cache DB functions
--
--====================================
local function cdbHGHasOperator(usrName)
    local sqlstr=string.format("select count(*) from tbl_hunting_group_operator"..
                               " where operator='%s'",usrName)
    local result = cdbQuery(sqlstr)    
    if result~=MSS_ERR_OBJ_FOUND then return -1 end

    result = cdbGetRowIntVal(0)
    if result<=0 then return -2 end

    return 1                           
end

local function cdbIsAutoOpertor(usrName)
    local sqlStr=string.format("select autoInd from tbl_hunting_group_operator" ..
                               " where operator='%s'",usrName)
    local result=cdbQuery(sqlStr)
    if result~=MSS_ERR_OBJ_FOUND then return MSS_FALSE end
    
    result=cdbGetRowIntVal(0)
    if result==MSS_YES then 
        return MSS_TRUE 
    else 
        return MSS_FALSE
    end
end

--====================================
--
-- Core functions
--
--====================================
local function operatorLogin()
    TraceInfo("\t operatorLogin")
    ssfHGOpertorLogin(fsm.operator)
end

local function operatorLogout()
    TraceInfo("\t operatorLogout")
    ssfHGOpertorLogout(fsm.operator)
end

--====================================
--
-- FSM common functions
--
--====================================
local function initParams()
    fsm.pfxLogin = ssfGetSysVarHGPrefixLogin()
    fsm.pfxLogout = ssfGetSysVarHGPrefixLogout()
    fsm.opType = EHG_OP_UNKNOWN
    fsm.operator = ""
end


local function checkIDP()
    TraceInfo("\t checkIDP")
    if fsm.profileID==MSS_INVALID_OBJ_ID then
        TraceInfo("\t\t cannot be invoked by outside user.")
        return -1
    end
    return 0
end


local function detectOperationType()
    TraceInfo("\t detectOperationType")
    if StrHasPrefix(fsm.calledNbr, fsm.pfxLogin)==MSS_TRUE then
        return EHG_OP_LOGIN
    elseif StrHasPrefix(fsm.calledNbr, fsm.pfxLogout)==MSS_TRUE then
        return EHG_OP_LOGOUT
    else
        return EHG_OP_UNKNOWN
    end
end


local function setOperatorStatus()
    TraceInfo("\t setOperatorStatus")
    if fsm.opType==EHG_OP_LOGIN then
        operatorLogin()
    else
        operatorLogout()
    end
end


local function playNotice()
    TraceInfo("\t playNotice")    
    SendCTR(CALLER_LEGID)

    local paArg = CreatePA()
    paArg.infoSend_annID_annID = HGOP_ANN_ID_NOTICE
    paArg.infoSend_numOfRepeat = 1
    SendPA(CALLER_LEGID, paArg)
end

--====================================
--
-- FSM state functions
--
--====================================
local function onIdleProc()
    TraceInfo("HGOperator:onIdleProc")
    initParams()

    local result=checkIDP()
    if result<0 then EndService() return end

    fsm.opType = detectOperationType()
    if fsm.opType==EHG_OP_UNKNOWN then
        TraceInfo("\t unknown operation type.")
        EndService()
        return
    end

    fsm.operator = fsm.callerNbr
    result = cdbHGHasOperator(fsm.operator)
    if result<0 then
        TraceInfo("\t cannot find operator: "..fsm.operator)
        EndService()
        return
    end
    
    result = cdbIsAutoOpertor(fsm.operator)
    if result==MSS_TRUE then
        TraceInfo("\t current operation cannot be applied to auto operator.")
        EndService()
        return
    end
    
    setOperatorStatus()
    playNotice()

    StartTimer(HGOP_TIMER_WAIT_ANN)
    EnterState(EHG_STATE_WAIT_ANN)
end

--====================================
--
-- Service main function
--
--====================================
function DestroyProc()
    TraceInfo("DestroyProc")
    SendReleaseCall()
end


function MainProc(currMsg)
    if fsm.state==EHG_STATE_IDLE then
        onIdleProc()
    elseif fsm.state==EHG_STATE_WAIT_ANN then
        EndService()
    else
        TraceInfo("unknown state: "..fsm.state)
        EndService()
    end
end    
