--===============================================
--
-- Common global functions for string procedures
--
--===============================================
require "utils.data_type"

function StrHasPrefix(origStr, destStr)    
    if origStr==nil or destStr==nil then return MSS_FALSE end

    local len1=string.len(origStr)
    local len2=string.len(destStr)
    if len1<len2 then return MSS_FALSE end

    local newStr = string.sub(origStr, 1, len2)
    if newStr==destStr then
        return MSS_TRUE
    else
        return MSS_FALSE
    end
end


function StrHasPostfix(origStr, destStr)
    if origStr==nil or destStr==nil then return MSS_FALSE end

    local len1=string.len(origStr)
    local len2=string.len(destStr)
    if len1<len2 then return MSS_FALSE end

    local startPos = len1 - len2 +1
    local newStr = string.sub(origStr, startPos, len1)
     if newStr==destStr then
         return MSS_TRUE
     else
         return MSS_FALSE
     end
end

-- cut 'destStr' from left side of 'origStr'
function StrLstrip(origStr, destStr)
    if MSS_FALSE==StrHasPrefix(origStr,destStr) then return origStr end

    local destLen=string.len(destStr)
    local newStr=string.sub(origStr, destLen+1)
    return StrLstrip(newStr, destStr)
end

-- cut 'destStr' from right side of 'origStr' 
function StrRstrip(origStr, destStr)
    if MSS_FALSE==StrHasPostfix(origStr, destStr) then return origStr end
 
    local len1=string.len(origStr)
    local len2=string.len(destStr)
    local newStr=string.sub(origStr, 1, len1-len2)
    return StrRstrip(newStr, destStr)
end


-- Split 'origStr' at string 'tokenStr', 
-- the first string is the left part, and the second string is the right part
-- both of them doesn't include the 'tokenStr'.
function StrLSplitAtStr(origStr, tokenStr)
    local start_i, end_j = string.find(origStr, tokenStr)
    if start_i==nil then return nil end
    local first_str = string.sub(origStr, 1, start_i-1)
    local secnd_str = string.sub(origStr, end_j+1)
    return first_str, secnd_str
end

function IntToHexStr(intVal)
    local strVal=string.format("0x%08X", intVal)
    return strVal
end

-- eof